

#define F_CPU 8000000L

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h> 
#include <math.h>
#include <avr/eeprom.h>


//   -     LCD
#include "bcd.h"

//  
#include "lcd_lib_2.h"

//  I2C
#include "i2c.h"

//  
#include "buttons.h"



//   
#define RTC_adr_write 0b11010000
#define RTC_adr_read  0b11010001

//  
unsigned char hour;
unsigned char min;
unsigned char sec;

//  
unsigned char day;
unsigned char wday;
unsigned char month;
unsigned char year;

//  
unsigned int  temp;
unsigned char t1;
unsigned char t2;

//  
unsigned char alarm_hour;
unsigned char alarm_min;
unsigned char alarm_on;

//  
unsigned char i=0;                  //   
unsigned char al_on;                //    ( / )
unsigned char al;                   //   
unsigned char j=1;                  //  


#define LED_PORT PORTB
#define LED_DDR DDRB
#define LED PB0


uint8_t alarm_hour_ee EEMEM = 0;
uint8_t alarm_min_ee EEMEM = 0;
uint8_t alarm_on_ee EEMEM = 0;


//      
const uint8_t alarm[] PROGMEM   = {0x02,0x04,0x04,0x0E,0x0E,0x1F,0x1F,0x04};



// ========================================================================================================================
// ========================================================================================================================
// DS3231



//    (   ) ====================================================
unsigned char bcd (unsigned char data)
{
	unsigned char bc;

	bc=((((data&(1<<6))|(data&(1<<5))|(data&(1<<4)))*0x0A)>>4)+((data&(1<<3))|(data&(1<<2))|(data&(1<<1))|(data&0x01));

	return bc;
}

unsigned char bin(unsigned char dec){

	char bcd;

	char n, dig, num, count;

	num = dec;
	count = 0;
	bcd = 0;
	for (n=0; n<4; n++) {
		dig = num%10;
		num = num/10;
		bcd = (dig<<count)|bcd;
		count += 4;
	}

	return bcd;
}


//    =======================================================================================================

//   
void RTC_init(void){

	i2c_start_cond();               //  i2c
	i2c_send_byte(RTC_adr_write);   //   ,  
	i2c_send_byte(0x0E);	        //    
	i2c_send_byte(0b00100000);      //       1 
	i2c_send_byte(0b00001000);      //   32 
	i2c_stop_cond();                //  i2c

}


//    
void RTC_read_time(void){

	i2c_start_cond();               //  i2c
	i2c_send_byte(RTC_adr_write);   //   ,  
	i2c_send_byte(0x00);	        //    
	i2c_stop_cond();                //  i2c

	i2c_start_cond();               //  i2c
	i2c_send_byte(RTC_adr_read);    //   ,  
	sec = bcd(i2c_get_byte(0));     //  , ACK
	min = bcd(i2c_get_byte(0));     //  , ACK
	hour = bcd(i2c_get_byte(0));    //  , ACK
	wday = bcd(i2c_get_byte(0));    //   , ACK
	day = bcd(i2c_get_byte(0));     //  , ACK
	month = bcd(i2c_get_byte(0));   //  , ACK
	year = bcd(i2c_get_byte(1));    //  , NACK
	i2c_stop_cond();                //  i2c

}


//  
void RTC_write_time(unsigned char hour1,unsigned char min1, unsigned char sec1){

    i2c_start_cond();               //  i2c
    i2c_send_byte(RTC_adr_write);   //   ,  
    i2c_send_byte(0x00);	        //    
    i2c_send_byte(bin(sec1));       // 0x00  (     ?)
	i2c_send_byte(bin(min1));       // 0x01 
    i2c_send_byte(bin(hour1));      // 0x02 
    i2c_stop_cond();                //  i2c

}


//  
void RTC_write_date(unsigned char wday, unsigned char day, unsigned char month, unsigned char year){

	i2c_start_cond();               //  i2c
    i2c_send_byte(RTC_adr_write);   //   ,  
    i2c_send_byte(0x03);	        //    
    i2c_send_byte(bin(wday));       // 0x03   ( - 1,  2,  3,  4,  5,  6,  7)
    i2c_send_byte(bin(day));        // 0x04  
	i2c_send_byte(bin(month));      // 0x05 
	i2c_send_byte(bin(year));       // 0x06 
    i2c_stop_cond();                //  i2c

}


//  
void RTC_read_temper(void){


	i2c_start_cond();               //  i2c
	i2c_send_byte(RTC_adr_write);   //   ,  
	i2c_send_byte(0x11);	        //    
	i2c_stop_cond();                //  i2c

	i2c_start_cond();               //  i2c
	i2c_send_byte(RTC_adr_read);    //   ,  
	t1 = i2c_get_byte(0);           //  MSB 
	t2 = i2c_get_byte(1);           //  LSB 
	i2c_stop_cond();                //  i2c

	t2=(t2>>7); 					//   6 -  0,25 (2 )
	                    			//   7 -  0,5 (1 )
	t2=t2*5;

}



//   
void put_wday (void) {
if (wday==1) {LCD_SendStr("Sun");}
if (wday==2) {LCD_SendStr("Mon");}
if (wday==3) {LCD_SendStr("Tue");}
if (wday==4) {LCD_SendStr("Wed");}
if (wday==5) {LCD_SendStr("Thu");}
if (wday==6) {LCD_SendStr("Fri");}
if (wday==7) {LCD_SendStr("Sat");}
}


// 
// ========================================================================================================================
// ========================================================================================================================



ISR (TIMER0_OVF_vect)               //      0
{

BUT_Debrief ();                     //  4 

}



int main (void)
{


// 
LCD_Init();
i2c_init();
BUT_Init();
RTC_init();


LED_DDR |= (1<<LED);
LED_PORT &= ~(1<<LED);



//   eeprom  
alarm_hour = eeprom_read_byte(&alarm_hour_ee);  
alarm_min = eeprom_read_byte(&alarm_min_ee); 
alarm_on = eeprom_read_byte(&alarm_on_ee);


LCD_SetUserChar(alarm, 1);      //      


//  0
TCCR0 |= 0b00000101;            //   1/1024
TIMSK |= 0b00000001;            //      T0.
TCNT0=0;                        //     T0 

sei();                          //   

main: 
LCD_Clear();





while (1)
{

i=BUT_GetKey();

//  
RTC_read_time ();

LCD_Goto(0,0);
BCD_2Lcd(hour);
LCD_WriteData(':');
BCD_2Lcd(min);
LCD_WriteData(':');
BCD_2Lcd(sec);

//  
LCD_Goto(0,1);
BCD_2Lcd(day);
LCD_WriteData('/');
BCD_2Lcd(month);
LCD_WriteData('/');
BCD_2Lcd(year);

//   
LCD_Goto(10,0);
put_wday ();

//  
RTC_read_temper();
LCD_Goto(10,1);
LCD_WriteData('+');
BCD_2Lcd(t1);
LCD_WriteData(',');
BCD_1Lcd(t2);

LCD_WriteData('C');

// 
if(alarm_on==1)       {
if(alarm_hour==hour)  {
if(alarm_min==min)    {
if(sec==1)            {
al=1;
}
}
}
}




//   
if (alarm_on==1) {LCD_Goto(15,0); LCD_WriteData(1);}
else {LCD_Goto(15,0); LCD_WriteData(' ');}


//    (     3)
i=BUT_GetKey();

if (i==3) 
{ al=0; }


//      
if (al==1) 
{

LED_PORT |= (1<<LED);

}

else
{

LED_PORT &=~(1<<LED);

}

//_delay_ms(200);




//===   =====================================================================================================
//==========================================================================================================================
//  1       
//  2    1 (   )
//  3     
//


if (i==1)
{

LCD_Clear();

while (3)                           //  
 {

    i=BUT_GetKey();

    if (i==1)
    {
      j++;
    }

    if (i==3)                       //   
    {
      i=0;
	  j=1;
      goto main;
    }

//   
if (j==1) {LCD_Goto(6,1); LCD_SendStr("^^");}
if (j==2) {LCD_Goto(6,1); LCD_SendStr("   ^^");}
if (j==3) {LCD_Goto(9,1); LCD_SendStr("   ^^");}

LCD_Goto(0,0);
LCD_SendStr("Alarm");

LCD_Goto(6,0);
BCD_2Lcd(alarm_hour);
LCD_WriteData(':');
BCD_2Lcd(alarm_min);
LCD_WriteData('-');
if (alarm_on==1)
{ LCD_SendStr("on "); }
else
{ LCD_SendStr("off"); }



    switch (j)
    {
       case (1):
	   {
       if (j!=1) {break;}
       if (i==2)
       {
         
         alarm_hour++;
         if (alarm_hour>23) {alarm_hour=0;}
		 
       }
	   }

       case (2):
	   {
       if (j!=2) {break;}
       if (i==2)
       {
         
         alarm_min++;
         if (alarm_min>59) {alarm_min=0;}
		 
       }
	   }

       case (3):
	   {
       if (j!=3) {break;}
       if (i==2)
       {
         
         alarm_on++;
         if (alarm_on>1) {alarm_on=0;}
		 
       }
	   }

    }


    if (j==4)                       //    
    {
	//   eeprom
	eeprom_write_byte(&alarm_hour_ee, alarm_hour);  
	eeprom_write_byte(&alarm_min_ee, alarm_min);  
	eeprom_write_byte(&alarm_on_ee, alarm_on);
      j=1;
      LCD_Clear();
      break;
    }



 }

while (2)                           //  
 {

    i=BUT_GetKey();

    if (i==1)
    {
      j++;
    }

    if (i==3)                       //   
    {
      i=0;
	  j=1;
      goto main;
    }

//   
if (j==1) {LCD_Goto(6,1); LCD_SendStr("^^");}
if (j==2) {LCD_Goto(6,1); LCD_SendStr("   ^^");}

LCD_Goto(0,0);
LCD_SendStr("Time");

LCD_Goto(6,0);
BCD_2Lcd(hour);
LCD_WriteData(':');
BCD_2Lcd(min);
LCD_WriteData(':');
BCD_2Lcd(0);


    switch (j)
    {
       case (1):
	   {
       if (j!=1) {break;}
       if (i==2)
       {
         
         hour++;
         if (hour>23) {hour=0;}
		 
       }
	   }

       case (2):
	   {
       if (j!=2) {break;}
       if (i==2)
       {
         
         min++;
         if (min>59) {min=0;}
		 
       }
	   }

    }


    if (j==3)                       //    
    {
	  RTC_write_time(hour, min, 0);
      j=1;
      LCD_Clear();
      break;
    }


 }

while (2)                           //  
 {

    i=BUT_GetKey();

    if (i==1)
    {
      j++;
    }

    if (i==3)                       //   
    {
      i=0;
	  j=1;
      goto main;
    }

//      
if (j==1) {LCD_Goto(5,1); LCD_SendStr("^^");}
if (j==2) {LCD_Goto(5,1); LCD_SendStr("   ^^");}
if (j==3) {LCD_Goto(8,1); LCD_SendStr("   ^^");}
if (j==4) {LCD_Goto(11,1); LCD_SendStr("   ^^");}

LCD_Goto(0,0);
LCD_SendStr("Date");

LCD_Goto(5,0);
BCD_2Lcd(day);
LCD_WriteData('/');
BCD_2Lcd(month);
LCD_WriteData('/');
BCD_2Lcd(year);
LCD_WriteData(' ');
put_wday();


    switch (j)
    {
       case (1):
	   {
       if (j!=1) {break;}
       if (i==2)
       {
         
         day++;
         if (day>31) {day=1;}
		 
       }
	   }

       case (2):
	   {
       if (j!=2) {break;}
       if (i==2)
       {
         
         month++;
         if (month>12) {month=1;}
		 
       }
	   }

       case (3):
	   {
       if (j!=3) {break;}
       if (i==2)
       {
         
         year++;
         if (year>40) {year=12;}
		 
       }
	   }

       case (4):
	   {
       if (j!=4) {break;}
       if (i==2)
       {
         
         wday++;
         if (wday>6) {wday=0;}
		 
       }
	   }

    } 


    if (j==5)                       //    
    {
	  RTC_write_date (wday, day, month, year);
      j=1;
      LCD_Clear();
      break;
    }



 }




}




//===   =======================================================================================================
//==========================================================================================================================




} // end while(1)
} // end main





